from PyQt5.QtCore import QSharedMemory, QSystemSemaphore


class SingleApplicationGuard:
    def __init__(self, key, data_buffer_size=1024):
        self.key = key
        self.shared_memory = QSharedMemory(self.key)
        self.semaphore = QSystemSemaphore(self.key + "_sem", 1)
        self.buffer_size = data_buffer_size

    def is_used(self):
        # 使用信号量确保线程安全
        self.semaphore.acquire()

        # 尝试附加到共享内存
        is_attached = self.shared_memory.attach()
        if is_attached:
            self.shared_memory.detach()

        # 尝试创建共享内存
        is_created = self.shared_memory.create(self.buffer_size)

        self.semaphore.release()

        # 如果无法创建且不是因为已经存在，则可能是其他错误
        if not is_created and self.shared_memory.error() == QSharedMemory.AlreadyExists:
            return True

        return not is_created

    def set_data(self, data: bytes):
        msg_length = len(data)
        if msg_length > self.buffer_size:
            raise Exception("Message Data Length [{0}] Over Buffer Size [{1}]".format(msg_length, self.buffer_size))
        is_lock = False
        try:
            is_lock = self.shared_memory.lock()
            if is_lock:
                dt_ptr = self.shared_memory.data()
                dt_ptr[:msg_length] = data
                return True
            return False
        except Exception as ex:
            raise ex
        finally:
            if is_lock:
                self.shared_memory.unlock()

    def get_data(self) -> bytes:
        is_attached = False
        try:
            is_attached = self.shared_memory.attach()
            if is_attached:
                dt_ptr = self.shared_memory.data()
                return bytes(dt_ptr).split(b'\x00')[0]
        except Exception as ex:
            raise ex
        finally:
            if is_attached:
                self.shared_memory.detach()




from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QWidget

from haige_pyqt.resources import get_about_pixmap
from haige_pyqt.uis.about import Ui_aboutFrm


class AboutFrom(QWidget, Ui_aboutFrm):
    def __init__(self, parent=None):
        super(AboutFrom, self).__init__(parent=parent)
        self.setupUi(self)
        self.setFixedSize(self.size())
        self.setWindowModality(Qt.ApplicationModal)
        self.setWindowFlags(Qt.WindowCloseButtonHint)
        self.lb_picture.setPixmap(get_about_pixmap())



from ctypes import wintypes

from haige_pyqt.straper.manager import WM_ACTIVATE_SELF_WINDOW_MESSAGE


def is_activate_window_message(message) -> bool:
    try:
        return wintypes.MSG.from_address(message.__int__()).message == WM_ACTIVATE_SELF_WINDOW_MESSAGE
    except:
        return False



from PyQt5.QtCore import pyqtSignal
from PyQt5.QtWidgets import QSlider


class QDoubleSlider(QSlider):
    doubleValueChanged = pyqtSignal(float)

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        # 默认范围0.0-1.0，步长0.01
        self._min = 0.0
        self._max = 1.0
        self._step = 0.01
        self._decimals = 2  # 小数位数

        # 内部使用整数表示，放大100倍以获得0.01精度
        self._factor = 10 ** self._decimals
        super().setMinimum(0)
        super().setMaximum(int((self._max - self._min) / self._step))
        super().setSingleStep(1)  # 内部步长为1对应0.01

        self.valueChanged.connect(self._emit_double_value)

    def _emit_double_value(self, value):
        self.doubleValueChanged.emit(self._min + value * self._step)

    def setMinimum(self, min_val):
        self._min = min_val
        self._update_range()

    def setMaximum(self, max_val):
        self._max = max_val
        self._update_range()

    def setStep(self, step):
        """设置步长，如0.01"""
        self._step = step
        self._update_range()

    def _update_range(self):
        """更新内部整数范围"""
        super().setMaximum(int((self._max - self._min) / self._step))

    def value(self):
        return round(self._min + super().value() * self._step, self._decimals)

    def setValue(self, value):
        value = max(self._min, min(self._max, value))
        int_value = int(round((value - self._min) / self._step))
        super().setValue(int_value)

#
# if __name__ == '__main__':
#     app = QApplication(sys.argv)
#     # 使用示例
#     slider = QDoubleSlider(Qt.Horizontal)
#     slider.setMinimum(0.0)
#     slider.setMaximum(1.0)
#     slider.setSingleStep(0.01)
#     slider.setValue(0.5)
#     slider.doubleValueChanged.connect(print)
#     slider.show()
#     sys.exit(app.exec_())



import sys
import win32con
import win32gui

from haige_pyqt.straper.guard import SingleApplicationGuard

WM_ACTIVATE_SELF_WINDOW_MESSAGE = win32con.WM_USER + 100


class ApplicationRunningManager:

    def __init__(self, app_id):
        self.single_app_lock = SingleApplicationGuard(app_id)

    def run(self, m_win_hwnd):
        if self.single_app_lock.is_used():
            self._active_started_app_window()
            sys.exit(0)
        self._set_main_win_hwndId(m_win_hwnd)

    def _set_main_win_hwndId(self, hwnd_id):
        self.single_app_lock.set_data(str(hwnd_id).encode('utf-8'))

    def _active_started_app_window(self):
        d_bts = self.single_app_lock.get_data().decode('utf-8')
        if d_bts + "" != "":
            hwnd = int(d_bts)
            win32gui.SendMessage(hwnd, WM_ACTIVATE_SELF_WINDOW_MESSAGE, 0, 0)



import os
import sys


def _get_app_dir():
    if getattr(sys, 'frozen', False):
        return os.path.dirname(sys.executable)
    elif __file__:
        return os.path.dirname(sys.argv[0])


App_Root = _get_app_dir()
App_Data = App_Root + "/dat"

if not os.path.exists(App_Data):
    os.mkdir(App_Data)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '.\haige_pyqt\uis\about.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_aboutFrm(object):
    def setupUi(self, aboutFrm):
        aboutFrm.setObjectName("aboutFrm")
        aboutFrm.resize(371, 365)
        self.label = QtWidgets.QLabel(aboutFrm)
        self.label.setGeometry(QtCore.QRect(20, 10, 91, 41))
        font = QtGui.QFont()
        font.setFamily("华文中宋")
        font.setPointSize(14)
        self.label.setFont(font)
        self.label.setObjectName("label")
        self.lb_picture = QtWidgets.QLabel(aboutFrm)
        self.lb_picture.setGeometry(QtCore.QRect(21, 64, 320, 260))
        self.lb_picture.setText("")
        self.lb_picture.setObjectName("lb_picture")

        self.retranslateUi(aboutFrm)
        QtCore.QMetaObject.connectSlotsByName(aboutFrm)

    def retranslateUi(self, aboutFrm):
        _translate = QtCore.QCoreApplication.translate
        aboutFrm.setWindowTitle(_translate("aboutFrm", "关于作者"))
        self.label.setText(_translate("aboutFrm", "联系作者"))



from __future__ import annotations

import sys

from PyQt5.QtCore import QObject, pyqtSignal
from PyQt5.QtWidgets import QToolButton, QAction, QPushButton


class ExclusiveCheckedGroup(QObject):
    on_checked_change = pyqtSignal(bool, QObject)

    def __init__(self, items: [QAction | QToolButton | QPushButton] = None, allow_empty_checked=True, parent=None):
        super().__init__(parent)
        self._items = []
        self.add(items)
        self._allow_empty_checked = allow_empty_checked
        self._ignore_toggled = False

    def add(self, items: [QAction | QToolButton | QPushButton]):
        if not items:
            return
        for item in items:
            if item not in self._items:
                item.setCheckable(True)
                item.toggled.connect(self._toggled)
                self._items.append(item)

    def remove(self, items: [QAction | QToolButton | QPushButton]):
        if not items:
            return
        for item in items:
            if item not in self._items:
                continue
            item.toggled.disconnect(self._toggled)
            self._items.remove(item)

    def unCheckedAll(self):
        self._ignore_toggled = True
        for item in self._items:
            item.setChecked(False)
        self._ignore_toggled = False

    def setChecked(self, item, checked=True):
        for _item in self._items:
            if item == _item:
                item.setChecked(checked)

    def _toggled(self, checked):
        if self._ignore_toggled:
            return
        has_checked = False
        sender = self.sender()
        if checked:
            for item in self._items:
                if item != sender:
                    item.setChecked(False)
        for ac in self._items:
            if ac.isChecked():
                has_checked = True
                break
        # 如果不允许空白选中（也就是必须要有一个选中，那 sender 就不能被设置为非选中状态）
        if not self._allow_empty_checked and not has_checked:
            has_checked = True
            sender.setChecked(True)
        self.on_checked_change.emit(has_checked, sender)


if __name__ == '__main__':
    from PyQt5.QtWidgets import QApplication, QMainWindow, QHBoxLayout, QFrame, QToolBar


    def _handle_on_checked_change(obj, sender):
        print(sender, sender.text(), obj)


    app = QApplication(sys.argv)
    main = QMainWindow()
    mainFram = QFrame()
    layout = QHBoxLayout(mainFram)
    mainFram.setLayout(layout)
    main.setCentralWidget(mainFram)

    ac1 = QAction(text="ac1")
    ac2 = QAction(text="ac2")
    ac3 = QAction(text="ac3")
    toolbar = QToolBar(main)
    toolbar.addAction(ac1)
    toolbar.addAction(ac2)
    toolbar.addAction(ac3)
    layout.addWidget(toolbar)
    ac_exclusive = ExclusiveCheckedGroup([ac1, ac2, ac3])
    ac_exclusive.on_checked_change.connect(_handle_on_checked_change)

    btn1 = QPushButton(text="btn1")
    btn2 = QPushButton(text="btn2")
    btn3 = QPushButton(text="btn3")
    layout.addWidget(btn1)
    layout.addWidget(btn2)
    layout.addWidget(btn3)
    ac_exclusive2 = ExclusiveCheckedGroup([btn1, btn2, btn3])
    ac_exclusive2.on_checked_change.connect(_handle_on_checked_change)

    tb1 = QToolButton(text="tb1")
    tb2 = QToolButton(text="tb2")
    tb3 = QToolButton(text="tb3")
    layout.addWidget(tb1)
    layout.addWidget(tb2)
    layout.addWidget(tb3)
    ac_exclusive3 = ExclusiveCheckedGroup([tb1, tb2, tb3])
    ac_exclusive3.on_checked_change.connect(_handle_on_checked_change)

    bt001 = QPushButton(text="must has on checked 01")
    tb001 = QToolButton(text="must has on checked 02")
    layout.addWidget(bt001)
    layout.addWidget(tb001)
    ac_exclusive_not_allow_empty_checked = ExclusiveCheckedGroup([bt001, tb001], allow_empty_checked=False)
    ac_exclusive_not_allow_empty_checked.on_checked_change.connect(_handle_on_checked_change)

    main.show()
    sys.exit(app.exec_())
